#include "MyShiftOut.h"
#include <WProgram.h>

int MyShiftOut::_clock;
int MyShiftOut::_latch;
int* MyShiftOut::_dataPins;
int MyShiftOut::_numGrps;
MyOutPin* MyShiftOut::firstInList;
MyFancyOut* MyShiftOut::firstFancy;

 
/********************************************************************************
 */
void MyShiftOut::init(int clock, int latch, int dataPins[], int numGrps)
{
  _clock = clock;
  _latch = latch;
  _dataPins = dataPins;
  _numGrps = numGrps;
  firstInList = 0;
  

  pinMode(_clock, OUTPUT);
  pinMode(_latch, OUTPUT);
  for(int i = 0; i < _numGrps; i++)
  {
    pinMode(_dataPins[i], OUTPUT);
  }
}

/********************************************************************************
 */
void MyShiftOut::addPin(MyOutPin* pin)
{
  pin->nextInList = firstInList;
  firstInList = pin;
}

/********************************************************************************
 */
void MyShiftOut::addFancyOut(MyFancyOut* fancy)
{
  fancy->nextFancy = firstFancy;
  firstFancy = fancy;
} 

/********************************************************************************
 */
void MyShiftOut::initCycle(MyOutPin* cycleStart)
{
  cycleStart->_currentState = true;
}

/********************************************************************************
 */
void MyShiftOut::inLoop()
{
  unsigned long tickCnt = millis();
  int data[_numGrps];
  boolean changed = false;
  int pwmProg;
  MyOutPin* pinPtr;
  MyFancyOut* fancyPtr;

  for(fancyPtr = firstFancy; fancyPtr; fancyPtr = fancyPtr->nextFancy)
  {
    fancyPtr->inLoop();
  }

  for(pinPtr = firstInList; pinPtr; pinPtr = pinPtr->nextInList)
  {
    // first set up primary action pins
    switch(pinPtr->_action)
    {
    case actOff:
      if(pinPtr->_currentState)
      {
        pinPtr->_currentState = false;
        changed = true;
      }
      break;

    case actOn:
      if(!pinPtr->_currentState)
      {
        pinPtr->_currentState = true;
        changed = true;
      }
      break;

    case actFlash:
      if(tickCnt > (pinPtr->_lastChange + pinPtr->_timing))
      {
        pinPtr->_currentState = (pinPtr->_currentState != true);
        pinPtr->_lastChange = tickCnt;
    
        changed = true;
      }
      break;
      
    case actCycle:
      if(pinPtr->_currentState)
      {
        if(tickCnt > (pinPtr->_lastChange + pinPtr->_timing))
        {
          pinPtr->_currentState = false;
          pinPtr->_otherPinOut->_currentState = true;
          pinPtr->_otherPinOut->_lastChange = tickCnt;
    
          changed = true;
        }
      }
      break;
    
    }
  }

  // not set up secondary action pins
  for(pinPtr = firstInList; pinPtr; pinPtr = pinPtr->nextInList)
  {
    switch(pinPtr->_action)
    {
    case actOpposite:
      if(pinPtr->_currentState == pinPtr->_otherPinOut->_currentState)
      {
        pinPtr->_currentState = !pinPtr->_otherPinOut->_currentState;
        changed = true;
      }
      break;

    case actSame:
      if(pinPtr->_currentState != pinPtr->_otherPinOut->_currentState)
      {
        pinPtr->_currentState = pinPtr->_otherPinOut->_currentState;
        changed = true;
      }
      break;
    }
  }

  // now do the pin updates (if required) 
  if(changed)
  {
    int i;
    digitalWrite(_latch, 0);
    for(int pass = 0; pass < 2; pass++)
    {
      for(i = 0; i < _numGrps; i++)
      {
        data[i] = 0;
      }
      for(pinPtr = firstInList; pinPtr; pinPtr = pinPtr->nextInList)
      {
        if((pinPtr->_chip == pass) && pinPtr->_currentState)
        {
          data[pinPtr->_pair] |= pinPtr->_hexVal;
        }
      }
      
      doTheOutput(data);
    }
    digitalWrite(_latch, 1);
  }
}

/********************************************************************************
 * the heart of the LED display
 */
void MyShiftOut::doTheOutput(int myDataOut[]) {
  // This shifts 8 bits out MSB first, 
  //on the rising edge of the clock,
  //clock idles low

  //internal function setup
  int i, x;
  int pinState;

  //clear everything out just in case to
  //prepare shift register for bit shifting
  for(x = 0; x < _numGrps; x++)
  {
    digitalWrite(_dataPins[x], 0);
  }
  digitalWrite(_clock, 0);

  //for each bit in the byte myDataOut
  for(i = 7; i >= 0; i--)
  {
    digitalWrite(_clock, 0);

    for(x = 0; x < _numGrps; x++)
    {
      //if the value passed to myDataOut and a bitmask result 
      // true then... so if we are at i=6 and our value is
      // %11010100 it would the code compares it to %01000000 
      // and proceeds to set pinState to 1.
      if(myDataOut[x] & (1 << i))
      {
        pinState= 1;
      }
      else
      {	
        pinState= 0;
      }
  
      //Sets the pin to HIGH or LOW depending on pinState
      digitalWrite(_dataPins[x], pinState);
    }
    //register shifts bits on upstroke of clock pin  
    digitalWrite(_clock, 1);
    //zero the data pin after shift to prevent bleed through
    for(x = 0; x < _numGrps; x++)
    {
      digitalWrite(_dataPins[1], 0);
    }
  }

  //stop shifting
  digitalWrite(_clock, 0);
}



