#ifndef MY_SHIFT_OUT_H
#define MY_SHIFT_OUT_H

const int actOff      = 0;
const int actOn       = 1;
const int actFlash    = 2;
const int actOpposite = 3;
const int actSame     = 4;
const int actCycle    = 5;


class MyShiftOut;

/********************************************************************************
 */
class MyOutPin
{
  friend class MyShiftOut;
public:
  void init(int pair, int chip, int pin);
  int getPair();
  int getChip();
  int getPin();
  
  void turnOn();
  void turnOff();
  void setFlash(int timing);
  void setSame(MyOutPin* asThis);
  void setOpposite(MyOutPin* asThis);
  void setCycle(MyOutPin* nextInCycle, int timing);
  int getAction();

private:
  int _pair;
  int _chip;
  int _pin;
  int _hexVal;
  
  int _action;
  int _timing; // milliseconds, blink = how long each flash
  MyOutPin* _otherPinOut;
  unsigned long _lastChange;
  bool _currentState; // is the light on or off
  
  MyOutPin* nextInList;
};

/********************************************************************************
 */
class MyFancyOut
{
  friend class MyShiftOut;
private:
  virtual void inLoop() = 0;
  MyFancyOut* nextFancy;
};

/********************************************************************************
 */
class MyKitt : public MyFancyOut
{
public:
  void setPins(MyOutPin* pinArray, int arrSize, int timing);
  void turnOff();
  void turnOn();

private:
  virtual void inLoop();
  MyOutPin* _pinArray;
  int _arrSize;
  int _timing;
  unsigned long _lastChange;
  bool goingForward;
  int currentPos;
  bool pausing;
};

/********************************************************************************
 */
class MyShiftOut
{
public:
  static void init(int clock, int latch, int dataPins[], int numGrps);
  static void inLoop();
  static void addPin(MyOutPin* pin);
  static void addFancyOut(MyFancyOut* fancy);
  static void initCycle(MyOutPin* cycleStart);

private:
  static void doTheOutput(int myDataOut[]);

  static int _clock;
  static int _latch;
  static int* _dataPins;
  static int _numGrps;
  static MyOutPin* firstInList;
  static MyFancyOut* firstFancy;
};

#endif MY_SHIFT_OUT_H
